/* 
   checkecho.c
   
   Check characters written to the serial port are echoed back.  Test
   for MP serial port I/O.
*/

#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* String function definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */

#include "captain/capser.h"

int fd;

#define N 26
#define RX_TOTAL 3200

int initport(int fd) {
	struct termios options;
	// Get the current options for the port...
	tcgetattr(fd, &options);
	// Set the baud rates to 19200...
	cfsetispeed(&options, B115200);
	cfsetospeed(&options, B115200);
	// Enable the receiver and set local mode...
	options.c_cflag |= (CLOCAL | CREAD);

	options.c_cflag &= ~PARENB;
	options.c_cflag &= ~CSTOPB;
	options.c_cflag &= ~CSIZE;
	options.c_cflag |= CS8;
	options.c_cflag &= ~ECHO;
	options.c_cflag &= ~ECHONL;

	if (options.c_cflag & ECHO)
	    printf("ECHO\n");
	if (options.c_cflag & ECHONL)
	    printf("ECHONL\n");

	// Set the new options for the port...
	cfmakeraw(&options);
	tcsetattr(fd, TCSANOW, &options);
	return 1;
}

int main(int argc, char **argv) {
    int i,j, nrx, rx_total, errors;
    char next_char, prev_char;
    char tx[N+1], rx[N+1];
        
    fd = open("/dev/ttyUSB0", O_RDWR | O_NOCTTY | O_NDELAY);
    if (fd == -1) {
	perror("open_port: Unable to open /dev/ttyUSB0 - ");
	return 1;
    } else {
	fcntl(fd, F_SETFL, 0);
    }
	
    printf("baud=%d\n", getbaud(fd));
    initport(fd);
    printf("baud=%d\n", getbaud(fd));

    for(j=0,i='A'; i<='Z'; j++,i++)
	tx[j] = i;
    tx[j] = 0;

    rx_total = errors = 0;
    while(rx_total < RX_TOTAL) {
	if (!writeport(fd, tx)) {
	    printf("write failed\n");
	    close(fd);
	    return 1;
	}

	//printf("written:%s\n", tx);
	
	//fcntl(fd, F_SETFL, FNDELAY); // don't block serial read

	nrx = read(fd, rx, N);
	for(i=0; i<nrx; i++) {
	    //printf("%c", rx[i]);
	    
	    if (rx_total) {
		/* check rx-ed correctly */

		if (prev_char != 'Z')
		    next_char = prev_char + 1;
		else
		    next_char = 'A';
		
		if (rx[i] != next_char) 
		    errors++;
	    }
	    prev_char = rx[i];
	}
	rx_total += nrx;
	   
    }
    printf("rx_total: %d  errors: %d\n", rx_total, errors);

    close(fd);
    return 0;
}
